Package: SeqVarTools
Version: 1.51.2
Type: Package
Title: Tools for variant data
Description: An interface to the fast-access storage format for VCF
        data provided in SeqArray, with tools for common operations and
        analysis.
Author: Stephanie M. Gogarten, Xiuwen Zheng, Adrienne Stilp
Maintainer: Stephanie M. Gogarten <sdmorris@uw.edu>
Depends: SeqArray
Imports: grDevices, graphics, stats, methods, Biobase, BiocGenerics,
        gdsfmt, GenomicRanges, IRanges, S4Vectors, GWASExactHW,
        logistf, Matrix, data.table,
Suggests: BiocStyle, RUnit, stringr, knitr, rmarkdown
License: GPL-3
URL: https://github.com/smgogarten/SeqVarTools
LazyData: yes
VignetteBuilder: knitr
biocViews: SNP, GeneticVariability, Sequencing, Genetics
Collate: AllClasses.R AllGenerics.R AllUtilities.R
        Methods-SeqVarGDSClass.R Methods-SeqVarData.R
        Methods-Iterator.R chromWithPAR.R duplicateDiscordance.R hwe.R
        inbreedCoeff.R mendelErr.R pca.R setVariantID.R
        alternateAlleleDetection.R refFrac.R regression.R
git_url: https://git.bioconductor.org/packages/SeqVarTools
git_branch: devel
git_last_commit: a22ff2d
git_last_commit_date: 2026-07-09
Repository: Bioconductor 3.24
Date/Publication: 2026-07-09
NeedsCompilation: no
Packaged: 2026-07-10 00:35:57 UTC; biocbuild
