importFrom(grDevices, adjustcolor)
importFrom(graphics, abline, plot, text)
importFrom(stats, as.formula, binom.test, binomial, coef,
           complete.cases, glm, lm, median, model.matrix,
           na.omit, pchisq, qchisq, setNames, vcov)

import(methods)
import(gdsfmt)
import(SeqArray)
importFrom(BiocGenerics, end)
importFrom(S4Vectors, elementMetadata, mcols, "mcols<-",
           queryHits, subjectHits)
importFrom(IRanges, IRanges, findOverlaps, mergeByOverlaps,
           subsetByOverlaps)
importFrom(GenomicRanges, GRanges, GRangesList, seqnames)
import(Biobase)
import(logistf)
import(data.table)
importFrom(GWASExactHW, HWExact)
importFrom(Matrix, Matrix)

exportClasses(SeqVarData,
              SeqVarIterator,
              SeqVarBlockIterator,
              SeqVarRangeIterator,
              SeqVarWindowIterator,
              SeqVarListIterator)

exportMethods(show,
              sampleData, "sampleData<-",
              variantData, "variantData<-",
              validateSex,
              granges,
              variantFilter,
              lastFilter, "lastFilter<-",
              variantBlock,
              variantRanges,
              iterateFilter,
              resetIterator,
              currentRanges,
              currentVariants,
              applyMethod,
              refChar,
              altChar,
              nAlleles,
              isVariant,
              isSNV,
              getGenotype,
              getGenotypeAlleles,
              refDosage,
              altDosage,
              alleleDosage,
              expandedAltDosage,
              expandedVariantIndex,
              variantInfo,
              getVariableLengthData,
              imputedDosage,
              titv,
              alleleFrequency,
              alleleCount,
              minorAlleleCount,
              missingGenotypeRate,
              heterozygosity,
              hethom,
              homozygosity,
              meanBySample,
              countSingletons,
              pca,
              hwe,
              inbreedCoeff,
              mendelErr,
              duplicateDiscordance,
              alternateAlleleDetection,
              refFrac,
              refFracOverHets,
              refFracPlot,
              regression,
              chromWithPAR)

export(SeqVarData,
       SeqVarIterator,
       SeqVarBlockIterator,
       SeqVarRangeIterator,
       SeqVarWindowIterator,
       SeqVarListIterator,
       setVariantID)
