# SeqVarTools
An interface to the fast-access storage format for VCF
data provided in [SeqArray](https://github.com/zhengxwen/SeqArray), with tools for common operations and
analysis.

## Bioconductor

[http://www.bioconductor.org/packages/release/bioc/html/SeqVarTools.html](http://www.bioconductor.org/packages/release/bioc/html/SeqVarTools.html)

* Tutorials
  * [Introduction to SeqVarTools](https://bioconductor.org/packages/release/bioc/vignettes/SeqVarTools/inst/doc/SeqVarTools.pdf)
  * [Iterators in SeqVarTools](https://bioconductor.org/packages/devel/bioc/vignettes/SeqVarTools/inst/doc/Iterators.pdf)
* [Reference manual](http://www.bioconductor.org/packages/release/bioc/manuals/SeqVarTools/man/SeqVarTools.pdf)
* [News](http://bioconductor.org/packages/release/bioc/news/SeqVarTools/NEWS)

## Installation

```R
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("SeqVarTools")
```
